"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.saveAsSVG = exports.saveAsGEXF = exports.saveAsJSON = exports.saveAsPNG = exports.renderAsDataURL = void 0;
const sigma_1 = __importDefault(require("sigma"));
const file_saver_1 = __importDefault(require("file-saver"));
const gexf = __importStar(require("graphology-gexf/browser"));
// @ts-ignore
const renderer_1 = __importDefault(require("graphology-svg/renderer"));
// @ts-ignore
const defaults_1 = require("graphology-svg/defaults");
// Taken and adapted from: https://github.com/jacomyal/sigma.js/blob/main/examples/png-snapshot/saveAsPNG.ts
function renderToAuxiliaryCanvas(renderer, inputLayers) {
    const { width, height } = renderer.getDimensions();
    // This pixel ratio is here to deal with retina displays.
    // Indeed, for dimensions W and H, on a retina display, the canvases
    // dimensions actually are 2 * W and 2 * H. Sigma properly deals with it, but
    // we need to readapt here:
    const pixelRatio = window.devicePixelRatio || 1;
    const tmpRoot = document.createElement('DIV');
    tmpRoot.style.width = `${width}px`;
    tmpRoot.style.height = `${height}px`;
    tmpRoot.style.position = 'absolute';
    tmpRoot.style.right = '101%';
    tmpRoot.style.bottom = '101%';
    document.body.appendChild(tmpRoot);
    // Instantiate sigma:
    const tmpRenderer = new sigma_1.default(renderer.getGraph(), tmpRoot, renderer.getSettings());
    // Copy camera and force to render now, to avoid having to wait the schedule /
    // debounce frame:
    tmpRenderer.getCamera().setState(renderer.getCamera().getState());
    tmpRenderer.refresh();
    // Create a new canvas, on which the different layers will be drawn:
    const canvas = document.createElement('CANVAS');
    canvas.setAttribute('width', width * pixelRatio + '');
    canvas.setAttribute('height', height * pixelRatio + '');
    const ctx = canvas.getContext('2d');
    // Draw a white background first:
    ctx.fillStyle = '#fff';
    ctx.fillRect(0, 0, width * pixelRatio, height * pixelRatio);
    // For each layer, draw it on our canvas:
    const canvases = tmpRenderer.getCanvases();
    const layers = inputLayers
        ? inputLayers.filter((id) => !!canvases[id])
        : Object.keys(canvases);
    layers.forEach((id) => {
        ctx.drawImage(canvases[id], 0, 0, width * pixelRatio, height * pixelRatio, 0, 0, width * pixelRatio, height * pixelRatio);
    });
    return [
        canvas,
        () => {
            // Cleanup:
            tmpRenderer.kill();
            tmpRoot.remove();
        },
    ];
}
function renderAsDataURL(renderer) {
    const [canvas, cleanup] = renderToAuxiliaryCanvas(renderer);
    const dataURL = canvas.toDataURL();
    cleanup();
    return dataURL;
}
exports.renderAsDataURL = renderAsDataURL;
function saveAsPNG(renderer) {
    const [canvas, cleanup] = renderToAuxiliaryCanvas(renderer);
    // Save the canvas as a PNG image:
    canvas.toBlob((blob) => {
        if (blob)
            file_saver_1.default.saveAs(blob, 'graph.png');
        cleanup();
    }, 'image/png');
}
exports.saveAsPNG = saveAsPNG;
function saveAsJSON(renderer) {
    const data = JSON.stringify(renderer.getGraph(), null, 2);
    file_saver_1.default.saveAs(new Blob([data], { type: 'application/json' }), 'graph.json');
}
exports.saveAsJSON = saveAsJSON;
function saveAsGEXF(renderer) {
    const data = gexf.write(renderer.getGraph());
    file_saver_1.default.saveAs(new Blob([data], { type: 'application/xml' }), 'graph.gexf');
}
exports.saveAsGEXF = saveAsGEXF;
function saveAsSVG(renderer) {
    const rendererSettings = renderer.getSettings();
    const settings = Object.assign({}, defaults_1.DEFAULTS);
    settings.nodes = {
        // @ts-ignore
        reducer: (_, n, a) => rendererSettings.nodeReducer(n, a),
        defaultColor: rendererSettings.defaultNodeColor,
    };
    settings.edges = {
        // @ts-ignore
        reducer: (_, e, a) => rendererSettings.edgeReducer(e, a),
        defaultColor: rendererSettings.defaultEdgeColor,
    };
    const data = (0, renderer_1.default)(renderer.getGraph(), settings);
    file_saver_1.default.saveAs(new Blob([data], { type: 'image/svg+xml' }), 'graph.svg');
}
exports.saveAsSVG = saveAsSVG;
//# sourceMappingURL=utils.js.map